package getWay

import (
	"encoding/json"
	"fmt"
	"git.168cad.top/zhengqiuyun/rym-util/log"
	"github.com/gin-gonic/gin"
	"net"
	"time"
)

func GetCurrentUserId(c *gin.Context) (uint32, bool) {
	userId, b := c.Get(CurrentUserIdKey)
	if b {
		transUserId := userId.(uint32)
		return transUserId, b
	}
	return 0, b
}

type UserInfo struct {
	Id      uint32
	Account string
	Name    string
}

func GetCurrentUser(c *gin.Context) *UserInfo {
	var userInfo = UserInfo{}
	adminResponse, b := c.Get(CurrentUserKey)
	if b {
		admin := adminResponse.(Admin)
		userInfo.Name = admin.Name
		userInfo.Account = admin.LoginAccount
		userInfo.Id = admin.Id
	}
	return &userInfo
}

type AdminResponse struct {
	code  int
	msg   string
	Admin Admin
}

type Admin struct {
	Id           uint32
	Name         string
	LoginAccount string
}

const requestBodyKey = "requestBody"

func BindRequestBody(c *gin.Context, param interface{}) error {
	c.ShouldBindJSON(param)
	bytes, err := json.Marshal(param)
	c.Set(requestBodyKey, string(bytes))
	return err
}

func GetParams(c *gin.Context) string {
	rd := ""
	params := c.Request.URL.Query().Encode()
	if params != "" {
		rd += params
	}
	ad := c.GetString(requestBodyKey)
	if ad != "" {
		rd += ad
	}
	return rd
}

func GetServerIP() string {
	addrs, err := net.InterfaceAddrs()
	if err != nil {
		return ""
	}
	for _, addr := range addrs {
		ipAddr, ok := addr.(*net.IPNet)
		if !ok {
			continue
		}
		if ipAddr.IP.IsLoopback() {
			continue
		}
		if !ipAddr.IP.IsGlobalUnicast() {
			continue
		}
		return ipAddr.IP.String()
	}
	return ""
}

func GetClientIP(c *gin.Context) string {
	ip := c.ClientIP()
	return ip
}

func GetURL(c *gin.Context) string {
	url := c.Request.URL.Path
	return url
}
func GetMethod(c *gin.Context) string {
	method := c.Request.Method
	return method
}
func SetStartTime(c *gin.Context) {
	c.Set(StartTimeKey, time.Now().UnixNano())
	logStart(c)
}
func GetStartTime(c *gin.Context) int64 {
	startTime := c.GetInt64(StartTimeKey)
	return startTime
}

func logStart(c *gin.Context) {
	log.Info(fmt.Sprintf("开始 %s %s %s %s", GetMethod(c), GetURL(c), GetToken(c), GetClientIP(c)))
}
